#include "gtdlink.h"

/************************************************************************\
*
* Module for circular doubly linked list with list head.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#ifndef NULL
#define NULL ((void *)0)
#endif

/************************************************************************\
*
* Initialize a link head.
*
\************************************************************************/
void GtDInit(GtDLink *dl)
{
	if(dl)
	{
		dl->next=dl;
		dl->prev=dl;
	}
}

/************************************************************************\
*
* Add a link after another link.
*
* IN: adl - Add link after this one
*     dl - Add this link
*
\************************************************************************/
void GtDAddAfter(GtDLink *adl,GtDLink *dl)
{
	if(adl && dl)
	{
		dl->next=adl->next;
		dl->prev=adl;
		adl->next->prev=dl;
		adl->next=dl;
	}
}

/************************************************************************\
*
* Add a link before another link.
*
* IN: bdl - Add before this one
*     dl - Add this link
*
\************************************************************************/
void GtDAddBefore(GtDLink *bdl,GtDLink *dl)
{
	if(bdl && dl)
	{
		dl->next=bdl;
		dl->prev=bdl->prev;
		bdl->prev->next=dl;
		bdl->prev=dl;
	}
}

/************************************************************************\
*
* Add a link at the beginning of a link head.
*
* IN: hdl - Link head
*     dl - Link to add
*
\************************************************************************/
void GtDAddHead(GtDLink *hdl,GtDLink *dl)
{
	if(hdl && dl)
	{
		dl->next=hdl->next;
		dl->prev=hdl;
		hdl->next->prev=dl;
		hdl->next=dl;
	}
}

/************************************************************************\
*
* Add link at the end of a link head.
*
* IN: hdl - Link head
*     dl - Link to add
*
\************************************************************************/
void GtDAddTail(GtDLink *hdl,GtDLink *dl)
{
	if(hdl && dl)
	{
		dl->next=hdl;
		dl->prev=hdl->prev;
		hdl->prev->next=dl;
		hdl->prev=dl;
	}
}

/************************************************************************\
*
* Remove a link form any position and return it if success.
* Only remove if not empty and then mark as empty.
*
\************************************************************************/
void *GtDRemove(GtDLink *dl)
{
	if(dl && (dl->next!=dl))
	{
		dl->next->prev=dl->prev;
		dl->prev->next=dl->next;
		dl->next=dl;
		dl->prev=dl;
		return(dl);
	}
	return(NULL);
}

/************************************************************************\
*
* Remove first link from a link head and return it if success.
*
\************************************************************************/
void *GtDRemHead(GtDLink *hdl)
{
	if(hdl && (hdl->next!=hdl))
	{
		hdl=hdl->next;
		hdl->next->prev=hdl->prev;
		hdl->prev->next=hdl->next;
		hdl->next=hdl;
		hdl->prev=hdl;
		return(hdl);
	}
	return(NULL);
}

/************************************************************************\
*
* Remove last link from a link head and return it if success.
*
\************************************************************************/
void *GtDRemTail(GtDLink *hdl)
{
	if(hdl && (hdl->prev!=hdl))
	{
		hdl=hdl->prev;
		hdl->next->prev=hdl->prev;
		hdl->prev->next=hdl->next;
		hdl->next=hdl;
		hdl->prev=hdl;
		return(hdl);
	}
	return(NULL);
}

/************************************************************************\
*
* Return first link from link head or NULL.
*
\************************************************************************/
void *GtDFirst(GtDLink *hdl)
{
	if(hdl && (hdl->next!=hdl))
		return(hdl->next);
	return(NULL);
}

/************************************************************************\
*
* Return last link from link head or NULL.
*
\************************************************************************/
void *GtDLast(GtDLink *hdl)
{
	if(hdl && (hdl->prev!=hdl))
		return(hdl->prev);
	return(NULL);
}

/************************************************************************\
*
* Return next link or NULL if link head.
*
\************************************************************************/
void *GtDNext(GtDLink *hdl,GtDLink *dl)
{
	if(hdl && dl && (dl->next!=hdl))
		return(dl->next);
	return(NULL);
}

/************************************************************************\
*
* Return previous link or NULL if link head.
*
\************************************************************************/
void *GtDPrev(GtDLink *hdl,GtDLink *dl)
{
	if(hdl && dl && (dl->prev!=hdl))
		return(dl->prev);
	return(NULL);
}

/************************************************************************\
*
* Return TRUE if link head is empty.
*
\************************************************************************/
int GtDEmpty(GtDLink *hdl)
{
	if(hdl && (hdl->next==hdl))
		return(1);
	return(0);
}
