#include <proto.h>

#include "gtsymbol.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtSymbol class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init symbol.
*
\************************************************************************/
GtSymbol *GtMkSymbol(char *n,GtManager *p,GtTag *tgs)
{
	GtSymbol *obj;
	
	if(!(obj=new GtSymbol()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtSymbol object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init symbol.
*
\************************************************************************/
GtSymbol *GtMkSymbol(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtSymbol *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkSymbol(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty symbol.
*
\************************************************************************/
GtSymbol::GtSymbol(void)
{
	stype=GTST_NONE;
	val=0;
	bdrw=1;
	minxs=7;
	minys=7;
}

/************************************************************************\
*
* Setup symbol.
*
\************************************************************************/
sbool GtSymbol::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_BDR,GTRT_BYTE,&bdrw);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Symbol:
				stype=t->d.val;
				if(stype>GTST_LAST_NR)
					stype=GTST_NONE;
				break;
			case GT_OnOff:
				if(t->d.val)
					val=TRUE;
				else
					val=FALSE;
				break;
			case GT_Direction:
				val=t->d.val;
				if(val>GTD_LAST_NR)
					val=GTD_DOWN;
				break;
			case GT_Border:
				bdrw=t->d.val;
				break;
		}
	}
	this->SetMinMaxSize(GTMAX(2*bdrw+5,7),GTMAX(2*bdrw+5,7),GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set symbol attributes.
*
\************************************************************************/
sbool GtSymbol::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE,bch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Symbol:
				stype=t->d.val;
				if(stype>GTST_LAST_NR)
					stype=GTST_NONE;
				ch=TRUE;
				break;
			case GT_OnOff:
				if(t->d.val)
					val=TRUE;
				else
					val=FALSE;
				ch=TRUE;
				break;
			case GT_Direction:
				val=t->d.val;
				if(val>GTD_LAST_NR)
					val=GTD_DOWN;
				ch=TRUE;
				break;
			case GT_Border:
				bdrw=t->d.val;
				bch=TRUE;
				break;
		}
	}
	if(bch)
		this->SetMinMaxSize(GTMAX(2*bdrw+5,7),GTMAX(2*bdrw+5,7),GTS_MAX,GTS_MAX);
	if(ch)
		this->Refresh();
	return(TRUE);
}
	
/************************************************************************\
*
* Get symbol attributes.
*
\************************************************************************/
void GtSymbol::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Symbol:
				*ptr=stype;
				break;
			case GT_OnOff:
			case GT_Direction:
				*ptr=val;
				break;
			case GT_Border:
				*ptr=bdrw;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtSymbol::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtSymbol")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set symbol event mask.
*
\************************************************************************/
void GtSymbol::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw symbol.
*
\************************************************************************/
sbool GtSymbol::Draw(XEvent *xe)
{
	int n,xp,yp;
	XPoint pts[4];
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(flags & GTWF_NOWIN)
	{
		xp=xpos;
		yp=ypos;
	}
	else
	{
		xp=0;
		yp=0;
	}
	switch(stype)
	{
		case GTST_TOGGLE:
			if(val)
			{
				gtfill->DrawRectangle(gtwin,xp,yp,xsize-1,ysize-1,TRUE);
				for(n=0; n<bdrw; n++)
				{
					GT3DBOX(gtdrk,gtlht,gtwin,xp+n,yp+n,xsize-2*n,ysize-2*n);
				}
			}
			else
			{
				for(n=0; n<bdrw; n++)
				{
					GT3DBOX(gtlht,gtdrk,gtwin,xp+n,yp+n,xsize-2*n,ysize-2*n);
				}
			}
			break;
		case GTST_RADIO:
			if(val)
			{
				pts[0].x=xp+xsize/2;
				pts[0].y=yp;
				pts[1].x=xp+xsize-1;
				pts[1].y=yp+ysize/2;
				pts[2].x=xp+xsize/2;
				pts[2].y=yp+ysize-1;
				pts[3].x=xp;
				pts[3].y=yp+ysize/2;
				gtfill->FillPolygon(gtwin,pts,4,Convex,CoordModeOrigin);
				for(n=0; n<bdrw; n++)
				{
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+n,xp+xsize-n-1,yp+ysize/2);
					gtlht->DrawLine(gtwin,xp+xsize-n-1,yp+ysize/2,xp+xsize/2,yp+ysize-n-1);
					gtlht->DrawLine(gtwin,xp+xsize/2,yp+ysize-n-1,xp+n,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp+n,yp+ysize/2,xp+xsize/2,yp+n);
				}
			}
			else
			{
				for(n=0; n<bdrw; n++)
				{
					gtlht->DrawLine(gtwin,xp+xsize/2,yp+n,xp+xsize-n-1,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp+xsize-n-1,yp+ysize/2,xp+xsize/2,yp+ysize-n-1);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+ysize-n-1,xp+n,yp+ysize/2);
					gtlht->DrawLine(gtwin,xp+n,yp+ysize/2,xp+xsize/2,yp+n);
				}
			}
			break;
		case GTST_SARROW:
			switch(val)
			{
				case GTD_LEFT:
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp,xp+xsize-1,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize-1,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize-1,yp);
					break;
				case GTD_RIGHT:
					gtlht->DrawLine(gtwin,xp,yp,xp,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp,yp+ysize-1,xp+xsize-1,yp+ysize/2);
					gtlht->DrawLine(gtwin,xp,yp,xp+xsize-1,yp+ysize/2);
					break;
				case GTD_UP:
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp,xp+xsize-1,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp,yp+ysize-1,xp+xsize-1,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp,yp+ysize-1,xp+xsize/2,yp);
					break;
				case GTD_DOWN:
					gtlht->DrawLine(gtwin,xp,yp,xp+xsize/2,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp,xp+xsize/2,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp,yp,xp+xsize-1,yp);
					break;
			}
			break;
		case GTST_CARROW:
			switch(val)
			{
				case GTD_LEFT:
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp,xp+xsize/2,yp+ysize/4);
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp+ysize/4,xp+xsize-1,yp+3*ysize/4);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+3*ysize/4,xp+xsize-1,yp+3*ysize/4);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+3*ysize/4,xp+xsize/2,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/2,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/2,yp);
					gtlht->DrawLine(gtwin,xp+xsize/2,yp+ysize/4,xp+xsize-1,yp+ysize/4);
					break;
				case GTD_RIGHT:
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp+ysize/2,xp+xsize/2,yp);
					gtlht->DrawLine(gtwin,xp+xsize/2,yp,xp+xsize/2,yp+ysize/4);
					gtlht->DrawLine(gtwin,xp,yp+ysize/4,xp,yp+3*ysize/4);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+3*ysize/4,xp,yp+3*ysize/4);
					gtlht->DrawLine(gtwin,xp+xsize/2,yp+3*ysize/4,xp+xsize/2,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp+ysize/2,xp+xsize/2,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp+xsize/2,yp+ysize/4,xp,yp+ysize/4);
					break;
				case GTD_UP:
					gtlht->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/2,yp);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp,xp+xsize-1,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp+xsize-1,yp+ysize/2,xp+3*xsize/4,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp+3*xsize/4,yp+ysize/2,xp+3*xsize/4,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp+3*xsize/4,yp+ysize-1,xp+xsize/4,yp+ysize-1);
					gtlht->DrawLine(gtwin,xp+xsize/4,yp+ysize-1,xp+xsize/4,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/4,yp+ysize/2);
					break;
				case GTD_DOWN:
					gtlht->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/2,yp+ysize-1);
					gtdrk->DrawLine(gtwin,xp+xsize/2,yp+ysize-1,xp+xsize-1,yp+ysize/2);
					gtlht->DrawLine(gtwin,xp+xsize-1,yp+ysize/2,xp+3*xsize/4,yp+ysize/2);
					gtdrk->DrawLine(gtwin,xp+3*xsize/4,yp+ysize/2,xp+3*xsize/4,yp);
					gtlht->DrawLine(gtwin,xp+3*xsize/4,yp,xp+xsize/4,yp);
					gtlht->DrawLine(gtwin,xp+xsize/4,yp,xp+xsize/4,yp+ysize/2);
					gtlht->DrawLine(gtwin,xp,yp+ysize/2,xp+xsize/4,yp+ysize/2);
					break;
			}
			break;
		case GTST_CYCLE:
			gtlht->DrawArc(gtwin,xp,yp,xsize-1,ysize-1,45*64,180*64,FALSE);
			gtdrk->DrawArc(gtwin,xp,yp,xsize-1,ysize-1,(180+45)*64,180*64,FALSE);
			break;
	}
	return(TRUE);
}
